@@ -27,6 +27,7 @@ use rtlola_hir::hir::ConcreteValueType;
 pub use rtlola_hir::hir::{
     InputReference, Layer, MemorizationBound, OutputReference, StreamLayers, StreamReference, WindowReference,
 };
+use serde::{Deserialize, Serialize};
 use uom::si::rational64::{Frequency as UOM_Frequency, Time as UOM_Time};
 use uom::si::time::nanosecond;
 
@@ -69,7 +70,7 @@ pub trait Stream {
 /// * [rtlola_frontend::parse](crate::parse) to obtain an [RtLolaMir] for a specification in form of a string or path to a specification file.
 /// * [rtlola_hir::hir::RtLolaHir] for a data structs designed for working _on_it.
 /// * [RtLolaAst](rtlola_parser::RtLolaAst), which is the most basic and down-to-syntax data structure available for RTLola.
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct RtLolaMir {
     /// Contains all input streams.
     pub inputs: Vec<InputStream>,
@@ -89,7 +90,7 @@ pub struct RtLolaMir {
 }
 
 /// Represents an RTLola value type.  This does not including pacing information, for this refer to [TimeDrivenStream] and [EventDrivenStream].
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
 pub enum Type {
     /// A boolean type
     Bool,
@@ -117,7 +118,7 @@ pub enum Type {
 }
 
 /// Represents an RTLola pacing type.
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
 pub enum PacingType {
     /// Represents a periodic pacing with a fixed frequency
     Periodic(UOM_Frequency),
@@ -128,7 +129,7 @@ pub enum PacingType {
 }
 
 #[allow(missing_docs)]
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
 pub enum IntTy {
     /// Represents an 8-bit integer.
     Int8,
@@ -140,7 +141,7 @@ pub enum IntTy {
     Int64,
 }
 #[allow(missing_docs)]
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
 pub enum UIntTy {
     /// Represents an 8-bit unsigned integer.
     UInt8,
@@ -153,7 +154,7 @@ pub enum UIntTy {
 }
 
 #[allow(missing_docs)]
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
 pub enum FloatTy {
     /// Represents a 32-bit floating point number.
     Float32,
@@ -184,7 +185,7 @@ impl From<ConcreteValueType> for Type {
 }
 
 /// Contains all information inherent to an input stream.
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Serialize, Deserialize)]
 pub struct InputStream {
     /// The name of the stream
     pub name: String,
@@ -205,7 +206,7 @@ pub struct InputStream {
 /// Contains all information relevant to every kind of output stream.
 ///
 /// Refer to [TimeDrivenStream], [EventDrivenStream], and [Trigger], as well as their respective fields in the Mir for additional information.
-#[derive(Debug, PartialEq, Clone)]
+#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
 pub struct OutputStream {
     /// The name of the stream.
     pub name: String,
@@ -232,7 +233,7 @@ pub struct OutputStream {
 /// A type alias for references to triggers.
 pub type TriggerReference = usize;
 /// Wrapper for output streams that are in-fact triggers.  Provides additional information specific to triggers.
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Serialize, Deserialize)]
 pub struct Trigger {
     /// The trigger message that is supposed to be conveyed to the user if the trigger reports a violation.
     pub message: String,
@@ -245,28 +246,18 @@ pub struct Trigger {
 }
 
 /// Information on the spawn and parametrization behavior of a stream
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct InstanceTemplate {
     /// Information on the spawn behavior of the stream
     pub spawn: SpawnTemplate,
     /// The condition under which the stream is not supposed to be evaluated
-    pub filter: Option<Expression>,
+    pub filter: FilterTemplate,
     /// The condition under which the stream is supposed to be closed
-    pub close: Option<Expression>,
-}
-
-impl Default for InstanceTemplate {
-    fn default() -> Self {
-        InstanceTemplate {
-            spawn: SpawnTemplate::default(),
-            filter: None,
-            close: None,
-        }
-    }
+    pub close: CloseTemplate,
 }
 
 /// Information on the spawn behavior of a stream
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct SpawnTemplate {
     /// The `target` expression needs to be evaluated whenever the stream with this SpawnTemplate is supposed to be spawned.  The result of the evaluation constitutes the respective parameters.
     pub target: Option<Expression>,
@@ -285,8 +276,45 @@ impl Default for SpawnTemplate {
     }
 }
 
+/// Information on the close behavior of a stream
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+pub struct CloseTemplate {
+    /// The `target` expression needs to be evaluated whenever the stream with this CloseTemplate is supposed to be closed.  The result of the evaluation constitutes whether the stream is closed.
+    pub target: Option<Expression>,
+    /// The timing of the close condition.
+    pub pacing: PacingType,
+    /// Indicates whether the close condition contains a reference to the stream it belongs to.
+    pub has_self_reference: bool,
+}
+impl Default for CloseTemplate {
+    fn default() -> Self {
+        CloseTemplate {
+            target: None,
+            pacing: PacingType::Constant,
+            has_self_reference: false,
+        }
+    }
+}
+
+/// Information on the close behavior of a stream
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+pub struct FilterTemplate {
+    /// The `target` expression needs to be evaluated whenever the stream with this FilterTemplate is supposed to be evaluated.  The result of the evaluation constitutes whether the stream is actually evaluated.
+    pub target: Option<Expression>,
+    /// The timing of the filter condition.
+    pub pacing: PacingType,
+}
+impl Default for FilterTemplate {
+    fn default() -> Self {
+        FilterTemplate {
+            target: None,
+            pacing: PacingType::Constant,
+        }
+    }
+}
+
 /// Wrapper for output streams providing additional information specific to time-driven streams.
-#[derive(Debug, PartialEq, Eq, Clone, Copy)]
+#[derive(Debug, PartialEq, Eq, Clone, Copy, Serialize, Deserialize)]
 pub struct TimeDrivenStream {
     /// A reference to the stream that is specified.
     pub reference: StreamReference,
@@ -318,7 +346,7 @@ impl TimeDrivenStream {
 }
 
 /// Wrapper for output streams providing additional information specific to event-based streams.
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Serialize, Deserialize)]
 pub struct EventDrivenStream {
     /// A reference to the stream that is specified
     pub reference: StreamReference,
@@ -327,7 +355,7 @@ pub struct EventDrivenStream {
 }
 
 /// Representation of the activation condition of event-based entities such as streams or spawn conditions
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Serialize, Deserialize)]
 pub enum ActivationCondition {
     /// Activate when all entries of the [Vec] are true.
     Conjunction(Vec<Self>),
@@ -340,7 +368,7 @@ pub enum ActivationCondition {
 }
 
 /// Represents an expression
-#[derive(Debug, PartialEq, Clone)]
+#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
 pub struct Expression {
     /// The kind and all kind-specific information of the expression
     pub kind: ExpressionKind,
@@ -349,7 +377,7 @@ pub struct Expression {
 }
 
 /// This enum contains all possible kinds of expressions and their relevant information.
-#[derive(Debug, PartialEq, Clone)]
+#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
 pub enum ExpressionKind {
     /// Load a constant value
     LoadConstant(Constant),
@@ -414,7 +442,7 @@ pub enum ExpressionKind {
 /// Represents a constant value of a certain kind.
 ///
 /// *Note* the type of the constant might be more general than the type of the constant.  For example, `Constant::UInt(3u64)` represents an RTLola UInt8 constant.
-#[derive(Debug, PartialEq, Clone)]
+#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
 pub enum Constant {
     #[allow(missing_docs)]
     Str(String),
@@ -429,7 +457,7 @@ pub enum Constant {
 }
 
 /// Arithmetical and logical operations
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
 pub enum ArithLogOp {
     /// Logic negation (!)
     Not,
@@ -478,7 +506,7 @@ pub enum ArithLogOp {
 }
 
 /// Represents an instance of a discrete window
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Serialize, Deserialize)]
 pub struct DiscreteWindow {
     /// The stream whose values will be aggregated
     pub target: StreamReference,
@@ -497,7 +525,7 @@ pub struct DiscreteWindow {
 }
 
 /// Represents an instance of a sliding window
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Serialize, Deserialize)]
 pub struct SlidingWindow {
     /// The stream whose values will be aggregated
     pub target: StreamReference,
@@ -515,7 +543,7 @@ pub struct SlidingWindow {
     pub ty: Type,
 }
 
-#[derive(Debug, PartialEq, Eq, Clone, Copy)]
+#[derive(Debug, PartialEq, Eq, Clone, Copy, Serialize, Deserialize)]
 /// The Ast representation of the different aggregation functions
 pub enum WindowOperation {
     /// Aggregation function to count the number of updated values on the accessed stream
@@ -664,6 +692,14 @@ impl RtLolaMir {
         }
     }
 
+    /// Provides immutable access to a stream.
+    pub fn stream(&self, reference: StreamReference) -> &dyn Stream {
+        match reference {
+            StreamReference::In(ix) => &self.inputs[ix],
+            StreamReference::Out(ix) => &self.outputs[ix],
+        }
+    }
+
     /// Produces an iterator over all stream references.
     pub fn all_streams(&self) -> impl Iterator<Item = StreamReference> {
         self.input_refs()
@@ -681,6 +717,16 @@ impl RtLolaMir {
         self.event_driven.iter().map(|t| self.output(t.reference)).collect()
     }
 
+    /// Return true if the specification contains any time-driven features.
+    /// This includes time-driven streams and time-driven spawn conditions.
+    pub fn has_time_driven_features(&self) -> bool {
+        !self.time_driven.is_empty()
+            || self
+                .outputs
+                .iter()
+                .any(|o| matches!(o.instance_template.spawn.pacing, PacingType::Periodic(_)))
+    }
+
     /// Provides a collection of all time-driven output streams.
     pub fn all_time_driven(&self) -> Vec<&OutputStream> {
         self.time_driven.iter().map(|t| self.output(t.reference)).collect()
@@ -714,18 +760,29 @@ impl RtLolaMir {
     }
 
     /// Provides a representation for the evaluation layers of all event-driven output streams.  Each element of the outer `Vec` represents a layer, each element of the inner `Vec` an output stream in the layer.
-    pub fn get_event_driven_layers(&self) -> Vec<Vec<OutputReference>> {
-        if self.event_driven.is_empty() {
+    pub fn get_event_driven_layers(&self) -> Vec<Vec<Task>> {
+        let mut event_driven_spawns = self
+            .outputs
+            .iter()
+            .filter(|o| matches!(o.instance_template.spawn.pacing, PacingType::Event(_)))
+            .peekable();
+
+        // Peekable is fine because the filter above does not have side effects
+        if self.event_driven.is_empty() && event_driven_spawns.peek().is_none() {
             return vec![];
         }
 
         // Zip eval layer with stream reference.
-        let streams_with_layers: Vec<(usize, OutputReference)> = self
+        let streams_with_layers = self
             .event_driven
             .iter()
             .map(|s| s.reference)
-            .map(|r| (self.output(r).eval_layer().into(), r.out_ix()))
-            .collect();
+            .map(|r| (self.output(r).eval_layer().into(), Task::Evaluate(r.out_ix())));
+
+        let spawns_with_layers =
+            event_driven_spawns.map(|o| (o.spawn_layer().inner(), Task::Spawn(o.reference.out_ix())));
+
+        let tasks_with_layers: Vec<(usize, Task)> = streams_with_layers.chain(spawns_with_layers).collect();
 
         // Streams are annotated with an evaluation layer. The layer is not minimal, so there might be
         // layers without entries and more layers than streams.
@@ -737,13 +794,13 @@ impl RtLolaMir {
         // e) If there are some, add them as layer.
 
         // a) Find the greatest layer. Maximum must exist because vec cannot be empty.
-        let max_layer = streams_with_layers.iter().max_by_key(|(layer, _)| layer).unwrap().0;
+        let max_layer = tasks_with_layers.iter().max_by_key(|(layer, _)| layer).unwrap().0;
 
         let mut layers = Vec::new();
         // b) For each potential layer
         for i in 0..=max_layer {
             // c) Find streams that would be in it.
-            let in_layer_i: Vec<OutputReference> = streams_with_layers
+            let in_layer_i: Vec<Task> = tasks_with_layers
                 .iter()
                 .filter_map(|(l, r)| if *l == i { Some(*r) } else { None })
                 .collect();
@@ -816,7 +873,7 @@ impl std::ops::Add for ValSize {
 }
 
 /// Representation of the different stream accesses
-#[derive(Debug, PartialEq, Clone, Copy)]
+#[derive(Debug, PartialEq, Clone, Copy, Serialize, Deserialize)]
 pub enum StreamAccessKind {
     /// Represents the synchronous access
     Sync,
@@ -837,7 +894,7 @@ pub enum StreamAccessKind {
 }
 
 /// Offset used in the lookup expression
-#[derive(Debug, PartialEq, Eq, Clone, Copy)]
+#[derive(Debug, PartialEq, Eq, Clone, Copy, Serialize, Deserialize)]
 pub enum Offset {
     /// A strictly positive discrete offset, e.g., `4`, or `42`
     Future(u32),
